"use strict";
const validator_1 = require("./validator");
const validation_1 = require("../error/validation");
class ItemValidator {
    constructor(item) {
        this.item = item;
    }
    isInt(field, min = Number.MIN_SAFE_INTEGER, max = Number.MAX_SAFE_INTEGER) {
        return validator_1.Validator.isInt(this.item[field], min, max);
    }
    escape(field) {
        return validator_1.Validator.escape(this.item[field]);
    }
    isString(field, min = 0, max = 256) {
        return validator_1.Validator.isString(this.item[field], min, max);
    }
    isEmail(field) {
        return validator_1.Validator.isEmail(this.item[field]);
    }
}
exports.ItemValidator = ItemValidator;
function wrap(fn) {
    return function (field) {
        try {
            return fn.apply(this, [].slice.call(arguments, 0));
        }
        catch (error) {
            const err = error instanceof validation_1.ValidationError ?
                error :
                new validation_1.ValidationError(validation_1.ValidationError.VALIDATION);
            err.details.invalidField = field;
            err.details.invalidValue = this.item[field];
            throw err;
        }
    };
}
Object.getOwnPropertyNames(ItemValidator.prototype).forEach((key) => {
    let value = ItemValidator.prototype[key];
    if (typeof value === 'function' && value.name !== 'constructor') {
        ItemValidator.prototype[key] = wrap(value);
    }
});
